@@ -24,10 +24,11 @@
 
 __author__ = "Dilshod Temirkhodjaev <tdilshod@gmail.com>"
 __license__ = "MIT"
-__version__ = "0.8.1"
+__version__ = "0.8.2"
 
-import csv, datetime, zipfile, sys, os, re, signal
+import csv, datetime, zipfile, sys, os, re, signal, io
 import xml.parsers.expat
+from decimal import Decimal
 from xml.dom import minidom
 
 try:
@@ -196,12 +197,26 @@ class Xlsx2csv:
         options.setdefault("skip_hidden_rows", True)
 
         self.options = options
+        self.py3 = sys.version_info[0] == 3
+        self.ziphandle = None
+
+        xlsxinputfile = None
+        if xlsxfile == "-" and self.py3:
+            xlsxfile = "STDIN"
+            if sys.stdin.buffer.seekable():
+                xlsxinputfile = sys.stdin.buffer
+            else:
+                xlsxinputfile = io.BytesIO(sys.stdin.buffer.read())
+        elif xlsxfile == "-" and not self.py3:
+            raise ValueError("The - notation for STDIN is not supported for python2")
+        else:
+            xlsxinputfile = xlsxfile
+
         try:
-            self.ziphandle = zipfile.ZipFile(xlsxfile)
+            self.ziphandle = zipfile.ZipFile(xlsxinputfile)
         except (zipfile.BadZipfile, IOError):
             raise InvalidXlsxFileException("Invalid xlsx file: " + str(xlsxfile))
 
-        self.py3 = sys.version_info[0] == 3
 
         self.content_types = self._parse(ContentTypes, "/[Content_Types].xml")
         self.shared_strings = self._parse(SharedStrings, self.content_types.types["shared_strings"])
@@ -215,8 +230,9 @@ class Xlsx2csv:
             self.shared_strings.escape_strings()
 
     def __del__(self):
-        # make sure to close zip file, ziphandler does have a close() method
-        self.ziphandle.close()
+        if self.ziphandle:
+            # make sure to close zip file
+            self.ziphandle.close()
 
     def getSheetIdByName(self, name):
         for s in self.workbook.sheets:
@@ -349,7 +365,7 @@ class Xlsx2csv:
                 if self.options['escape_strings'] and sheet.filedata:
                     sheet.filedata = re.sub(r"(<v>[^<>]+)&#10;([^<>]+</v>)", r"\1\\n\2",
                                             re.sub(r"(<v>[^<>]+)&#9;([^<>]+</v>)", r"\1\\t\2",
-                                                   re.sub(r"(<v>[^<>]+)&#13;([^<>]+</v>)", r"\1\\r\2", sheet.filedata)))
+                                                   re.sub(r"(<v>[^<>]+)&#13;([^<>]+</v>)", r"\1\\r\2", sheet.filedata.decode())))
                 sheet.to_csv(writer)
             finally:
                 sheet_file.close()
@@ -789,17 +805,15 @@ class Sheet:
         if self.in_cell_value:
             format_type = None
             format_str = "general"
-            self.collected_string += data
-            self.data = self.collected_string
+            self.data += data
             if self.colType == "s":  # shared string
                 format_type = "string"
-                self.data = self.sharedStrings[int(self.data)]
+                self.data = self.sharedStrings[int(data)]
             elif self.colType == "b":  # boolean
                 format_type = "boolean"
                 self.data = (int(data) == 1 and "TRUE") or (int(data) == 0 and "FALSE") or data
             elif self.colType == "str" or self.colType == "inlineStr":
                 format_type = "string"
-                self.data = data
             elif self.s_attr:
                 s = int(self.s_attr)
 
@@ -833,7 +847,7 @@ class Sheet:
             elif self.colType == "n":
                 format_type = "float"
 
-            if format_type and not format_type in self.ignore_formats:
+            if format_type and not format_type in self.ignore_formats and self.data != "#N/A":
                 try:
                     if format_type == 'date':  # date/time
                         if self.workbook.date1904:
@@ -857,22 +871,28 @@ class Sheet:
                         t = int(round((float(self.data) % 1) * 24 * 60 * 60, 6))  # it should be in seconds
                         d = datetime.time(int((t // 3600) % 24), int((t // 60) % 60), int(t % 60))
                         self.data = d.strftime(self.timeformat)
-                    elif format_type == 'float' and ('E' in self.data or 'e' in self.data):
-                        self.data = str(self.floatformat or '%f') % float(self.data)
-                    # if cell is general, be aggressive about stripping any trailing 0s, decimal points, etc.
-                    elif format_type == 'float' and format_str == 'general':
-                        self.data = ("%f" % (float(self.data))).rstrip('0').rstrip('.')
-                    elif format_type == 'float' and format_str[0:3] == '0.0':
-                        if self.floatformat:
-                            self.data = str(self.floatformat) % float(self.data)
-                        else:
-                            L = len(format_str.split(".")[1])
-                            if '%' in format_str:
-                                L += 1
-                            self.data = ("%." + str(L) + "f") % float(self.data)
                     elif format_type == 'float':
-                        # unsupported float formatting
-                        self.data = ("%f" % (float(self.data))).rstrip('0').rstrip('.')
+                        data = float(self.data)
+                        if not self.floatformat and data.is_integer():
+                            # repr(float(...)) - workaround to correctly round precision for floats
+                            # repr gives same result on python 2 and 3, while str is different on python 2
+                            self.data = "%i" % Decimal(repr(float(self.data)))
+                        elif ('E' in self.data or 'e' in self.data):
+                            self.data = str(self.floatformat or '%f') % data
+                        # if cell is general, be aggressive about stripping any trailing 0s, decimal points, etc.
+                        elif format_str == 'general':
+                            self.data = ("%f" % data).rstrip('0').rstrip('.')
+                        elif format_str[0:3] == '0.0':
+                            if self.floatformat:
+                                self.data = str(self.floatformat) % data
+                            else:
+                                L = len(format_str.split(".")[1])
+                                if '%' in format_str:
+                                    L += 1
+                                self.data = ("%." + str(L) + "f") % data
+                        else:
+                            # unsupported float formatting
+                            self.data = ("%f" % data).rstrip('0').rstrip('.')
 
                 except (ValueError, OverflowError):  # this catch must be removed, it's hiding potential problems
                     raise XlsxValueError("Error: potential invalid date format.")
@@ -890,10 +910,8 @@ class Sheet:
                 self.colIndex += 1
             self.data = ""
             self.in_cell = True
-        elif self.in_cell and (
-                    (name == 'v' or name == 'is') or (has_namespace and (name.endswith(':v') or name.endswith(':is')))):
+        elif self.in_cell and ((name == 'v' or name == 't') or (has_namespace and name.endswith(':v'))):
             self.in_cell_value = True
-            self.collected_string = ""
         elif self.in_sheet and (name == 'row' or (has_namespace and name.endswith(':row'))) and ('r' in attrs) and not (self.skip_hidden_rows and 'hidden' in attrs and attrs['hidden'] == '1'):
             self.rowNum = attrs['r']
             self.in_row = True
@@ -903,9 +921,6 @@ class Sheet:
             self.spans = None
             if 'spans' in attrs:
                 self.spans = [int(i) for i in attrs['spans'].split(" ")[-1].split(":")]
-        elif name == 't':
-            # reset collected string
-            self.collected_string = ""
 
         elif name == 'sheetData' or (has_namespace and name.endswith(':sheetData')):
             self.in_sheet = True
@@ -923,8 +938,7 @@ class Sheet:
 
     def handleEndElement(self, name):
         has_namespace = name.find(":") > 0
-        if self.in_cell and ((name == 'v' or name == 'is' or name == 't') or (
-                    has_namespace and (name.endswith(':v') or name.endswith(':is')))):
+        if self.in_cell and ((name == 'v' or name == 't') or (has_namespace and name.endswith(':v'))):
             self.in_cell_value = False
         elif self.in_cell and (name == 'c' or (has_namespace and name.endswith(':c'))):
             t = 0
@@ -956,7 +970,7 @@ class Sheet:
                             val = val.encode("utf-8")
                         d.append(val)
                 else:
-                    d = [""] * (max(self.columns.keys()) + 1)
+                    d = [""] * (max(self.columns_count, max(self.columns.keys()) + 1))
                     for k in self.columns.keys():
                         val = self.columns[k]
                         if not self.py3:
@@ -966,6 +980,8 @@ class Sheet:
                     l = self.spans[1]
                     if len(d) < l:
                         d += (l - len(d)) * ['']
+                if self.columns_count < 0:
+                    self.columns_count = len(d)
 
                 # write empty lines
                 if not self.skip_empty_lines:
@@ -1051,7 +1067,7 @@ def main():
 
     if "ArgumentParser" in globals():
         parser = ArgumentParser(description="xlsx to csv converter")
-        parser.add_argument('infile', metavar='xlsxfile', help="xlsx file path")
+        parser.add_argument('infile', metavar='xlsxfile', help="xlsx file path, use '-' to read from STDIN")
         parser.add_argument('outfile', metavar='outfile', nargs='?', help="output csv file path")
         parser.add_argument('-v', '--version', action='version', version=__version__)
         nargs_plus = "+"
@@ -1132,6 +1148,8 @@ def main():
         options.delimiter = ','
     elif options.delimiter[0] == 'x':
         options.delimiter = chr(int(options.delimiter[1:]))
+    elif options.delimiter == 'fs' or options.delimiter == 'FS':
+        options.delimiter = chr(28)
     else:
         sys.exit("error: invalid delimiter\n")
 
@@ -1198,6 +1216,8 @@ def main():
     try:
         if os.path.isdir(options.infile):
             convert_recursive(options.infile, sheetid, outfile, kwargs)
+        elif not os.path.exists(options.infile) and options.infile != "-":
+            raise InvalidXlsxFileException("Input file not found!")
         else:
             xlsx2csv = Xlsx2csv(options.infile, **kwargs)
             if options.sheetname:
